;; Per-pixel diffuse lighting shader with texture example (adapted from <a href="http://www.rastertek.com/dx11tut09.html)" target="_blank">http://www.rastertek.com/dx11tut09.html)</a>

INCDIR "../../samples/images/"

DebugLoud
;UseDirectX11
Graphics3D 800,600,32,2

; shaders init
vs=RecallVertexShader("vertex_shader")
ps=RecallPixelShader("pixel_shader")

AddShaderConstant vs,"worldMatrix",15
AddShaderConstant vs,"viewMatrix",17
AddShaderConstant vs,"projectionMatrix",16
ValidateConstants vs

; scene
cam=CreateCamera(cam)
CameraClsColor cam,100,100,100

cube=CreateCube()
MoveEntity cube,2,0,5

sph=CreateSphere(16)
MoveEntity sph,-2,0,5

bnk=RecallFileFromDir("../../samples/images/","leaf.jpg")

tex=LoadTextureFromBank(bnk,9)

EntityTexture cube,tex
EntityTexture sph,tex

; apply shaders to entities
EntityVertexShader cube,vs
EntityPixelShader cube,ps

EntityVertexShader sph,vs
EntityPixelShader sph,ps


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Repeat

TurnEntity cube,.004,.01,0
TurnEntity sph,0,0.01,0

RenderWorld

Flip 0

Until KeyDown(1)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
End


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; VERTEX SHADER
Script vertex_shader("vs_main",1,0)
/*
Test comment
*/

cbuffer constants
{
	float4x4 worldMatrix;
	float4x4 viewMatrix;
	float4x4 projectionMatrix;
}

struct VS_OUTPUT
{
	float4 position : SV_POSITION;
	float4 position0 : POSITION2;
	float4 Color : COLOR;
	float3 normal : NORMAL;
	float2 tex : TEXCOORD0;
};

struct VS_INPUT
{
	float3 position : POSITION;
	float3 normal : NORMAL;
	float4 color : COLOR0;
	float2 tc : TEXCOORD0;
	float2 tc2 : TEXCOORD1;
	float4 instance_m0 : TEXCOORD2;
	float4 instance_m1 : TEXCOORD3;
	float4 instance_m2 : TEXCOORD4;
	float4 instance_m3 : TEXCOORD5;
	float4 Color2 : COLOR1;
};

//---------------------------------------------------------------------
// Name: vs_main()
// Desc: The vertex shader 
//---------------------------------------------------------------------
VS_OUTPUT vs_main(VS_INPUT input)
{
	VS_OUTPUT output;
	output.position = float4(input.position,1);

	// Update normals
	//output.normal = input.normal;
	output.normal = mul(input.normal, (float3x3)worldMatrix);
	
	output.position0 = output.position;

	output.Color = input.color;

	// Calculate the position of the vertex against the world, view, and projection matrices.
	output.position = mul(output.position, worldMatrix);
	output.position = mul(output.position, viewMatrix);
	output.position = mul(output.position, projectionMatrix);

	// Set the texture
	output.tex = input.tc;
	return output;
}
End Script

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PIXEL SHADER
Script pixel_shader("ps_main",2,0)

Texture2D shaderTexture;
SamplerState SampleType;

struct VS_OUTPUT
{
	float4 position : SV_POSITION;
	float4 position0 : POSITION2;
	float4 Color : COLOR;
	float3 normal : NORMAL;
	float2 tex : TEXCOORD0;
};

float4 ps_main(VS_OUTPUT Input) : SV_TARGET
{

	// light parameters
	float3 lightDirection = float3(-0.5,-1,0.5);
	float4 diffuseColor = float4(1,1,1,1);
	float4 ambientColor = float4(1,1,1,1);
	float diffuseIntensity = 1;
	float ambientIntensity = 0.2;

	// variables
	float4 color;
    float4 textureColor;
	float3 lightDir;
    float lightIntensity;

	// pixel color from texture
	textureColor = shaderTexture.Sample(SampleType, Input.tex);

	// apply ambient light first
	color = ambientColor*ambientIntensity;

    // invert the light direction for calculations
    lightDir = -lightDirection;

	// normalize the light direction to avoid to affect the light intensity
	lightDir = normalize(lightDir); // avoid to affect light intensity

    // calculate the amount of light on this pixel
    lightIntensity = dot(Input.normal, lightDir);
	
	if(lightIntensity > 0.0f)
	{
        // determine the final diffuse color based on the diffuse color and the amount of light intensity
        color += (diffuseColor * diffuseIntensity * lightIntensity);
    }

	// merge with texture color
	color = color * textureColor;

	// normalize the color to avoid sursaturation
	color = saturate(color);

	// set the pixel alpha to maximum opacity (if not the pixel is transparent ??)
	color.a = 1;

    return color;

}

End Script