
Type FunctionData
	Field bmxfunc$
	Field dllfunc$
End Type

Const KEY_WORDS = 10

Dim keyword$ (KEY_WORDS)

	keyword$ (0) = "global"
	keyword$ (1) = "type"
	keyword$ (2) = "max"
	keyword$ (3) = "min"
	keyword$ (4) = "string"
	keyword$ (5) = "method"
	keyword$ (6) = "double"
	keyword$ (7) = "float"
	keyword$ (8) = "int"
	keyword$ (9) = "local"

Function ReplaceMaxKeywords$ (bmxfunc$)

	firstbracket = Instr (bmxfunc$, "(")
	secondbracket = Instr (bmxfunc$, ")", firstbracket + 1)

	If (firstbracket > 0) And (secondbracket > 0)

		funcname$ = Left (bmxfunc$, firstbracket - 1)

		If (Right (funcname$, 1) <> "%") And (Right (funcname$, 1) <> "#") And (Right (funcname$, 1) <> "$")
			funcname$ = funcname$ + "%"
		EndIf
		
		firstbracket = firstbracket + 1
		parameters$ = Mid (bmxfunc$, firstbracket, secondbracket - firstbracket)
		
		If parameters$ <> ""
			
			; Reconstructing bmxfunc$ only if it has parameters...
		
			bmxfunc$ = funcname$ + "("
	
			parameters$ = Trim (parameters$) ; Ditch any spaces/tabs that might creep in
			
			comma = Instr (parameters$, ",")
			
			While comma
			
				param$ = Left (parameters$, comma - 1)

				If (Right (param$, 1) <> "%") And (Right (param$, 1) <> "#") And (Right (param$, 1) <> "$")
					param$ = param$ + "%"
				EndIf
		
				; Take out parameter type for name-check...
				
				tparam$ = Replace (param$, "%", "")
				tparam$ = Replace (tparam$, "#", "")
				tparam$ = Replace (tparam$, "$", "")
			
				parameters$ = Mid (parameters$, comma + 1)
				
				For check = 0 To KEY_WORDS - 1
					If tparam$ = keyword$ (check)
						param$ = "_" + param$
					EndIf
				Next

				bmxfunc$ = bmxfunc$ + param$ + ","
				
				comma = Instr (parameters$, ",")
				
			Wend
			
			param$ = parameters$

			If (Right (param$, 1) <> "%") And (Right (param$, 1) <> "#") And (Right (param$, 1) <> "$")
				param$ = param$ + "%"
			EndIf
				
			tparam$ = Replace (param$, "%", "")
			tparam$ = Replace (tparam$, "#", "")
			tparam$ = Replace (tparam$, "$", "")

			For check = 0 To KEY_WORDS - 1
				If tparam$ = keyword$ (check)
					param$ = "_" + param$
				EndIf
			Next

			bmxfunc$ = bmxfunc$ + param$ + ")"
			
		EndIf
		
	EndIf

	Return bmxfunc$
	
End Function

; -------------------------------------------------------
; Main program!
; -------------------------------------------------------

dll$ = ""

file = ReadFile ("hardwired.decls")

If file

	While Not Eof (file)
	
		bb$ = ReadLine (file)
		
		If bb$ <> ""
		
			; -----------------------------------------------------------------
			; Find DLL...
			; -----------------------------------------------------------------

			If Lower (Left (Trim (bb$), 4)) = ".lib" ; ADDED Trim!
				
				firstquote = Instr (bb$, Chr (34))
				
				firstquote = firstquote + 1 ; Get past first quote
				
				If firstquote
					secondquote = Instr (bb$, Chr (34), firstquote + 1)
					dll$ = Mid (bb$, firstquote, secondquote - firstquote)
					If dll$ = ""
						Print "No DLL found!"
						Exit
					EndIf
				Else
					Print "No DLL line!"
					Exit
				EndIf
			
			; -----------------------------------------------------------------
		
			Else
			
				; -------------------------------------------------------------	
				; Standard decl line...
				; -------------------------------------------------------------	
				
				; Basic structure:
				
				; function | optional return type | opening bracket | params | closing bracket | colon | quoted real function name
				
				split = Instr (bb$, ":")
				
				bmxfunc$ = ""
				dllfunc$ = ""
				
				If split

					fd.FunctionData = New FunctionData

					bmxfunc$ = Left (bb$, split - 1)

					bmxfunc$ = ReplaceMaxKeywords (bmxfunc$)
					
					bmxfunc$ = Replace (bmxfunc$, "%", ":Int")
					bmxfunc$ = Replace (bmxfunc$, "#", ":Float")
					bmxfunc$ = Replace (bmxfunc$, "$", ":Byte Ptr")

					If bmxfunc$ <> ""
						fd\bmxfunc$ = bmxfunc$
					EndIf

					dllfunc$ = Mid (bb$, split + 1)

					firstquote = Instr (dllfunc$, Chr (34))
					
					firstquote = firstquote + 1 ; Get past first quote
					
					If firstquote
					
						secondquote = Instr (dllfunc$, Chr (34), firstquote + 1)
						
						dllfunc$ = Mid (dllfunc$, firstquote, secondquote - firstquote)

						If dllfunc$ <> ""
							fd\dllfunc$ = dllfunc$
						EndIf
	
					Else
						Print "No quoted DLL function!"
						Exit
					EndIf
					
				Else
					Print "Not taken lines without colons into account! Doh!"
					Exit
				EndIf
				
			EndIf

		EndIf
	
	Wend
	
	CloseFile file
	
EndIf

bmxfile = WriteFile ("hardwiredlib.bmx")

If bmxfile

	WriteLine bmxfile, ""
	WriteLine bmxfile, "Local hardwired_dll:String = " + Chr (34) + dll$ + Chr (34)
	WriteLine bmxfile, ""

	WriteLine bmxfile, "Local hardwiredlib:Int = LoadLibraryA (hardwired_dll)"
	WriteLine bmxfile, ""
	WriteLine bmxfile, "If Not hardwiredlib"
	WriteLine bmxfile, Chr (9) + "RuntimeError " + Chr (34) + "Place " + Chr (34) + " + hardwired_dll + " + Chr (34) + " in same folder as this program!" + Chr (34)
	WriteLine bmxfile, "EndIf"
	WriteLine bmxfile, ""

	WriteLine bmxfile, "Global missing_functions:TList = CreateList ()"
	WriteLine bmxfile, ""
	WriteLine bmxfile, "Function ListMissingFunctions ()"
	WriteLine bmxfile, ""
	WriteLine bmxfile, Chr (9) + "Print " + Chr (34) + Chr (34)
	WriteLine bmxfile, Chr (9) + "Print " + Chr (34) + "Missing functions:" + Chr (34)
	WriteLine bmxfile, Chr (9) + "Print " + Chr (34) + Chr (34)
	WriteLine bmxfile, ""
	WriteLine bmxfile, Chr (9) + "For Local missing:String = EachIn missing_functions"
	WriteLine bmxfile, Chr (9) + Chr (9) + "Print " + Chr (34) + "~t" + Chr (34) + " + missing"
	WriteLine bmxfile, Chr (9) + "Next"
	WriteLine bmxfile, ""
	WriteLine bmxfile, "End Function"
	WriteLine bmxfile, ""

	For fd.FunctionData = Each FunctionData
		WriteLine bmxfile, "Global " + fd\bmxfunc$ + " " + Chr (34) + "win32" + Chr (34)
	Next
	
	WriteLine bmxfile, ""

	For fd.FunctionData = Each FunctionData
	
		func$ = fd\bmxfunc$
		
		bracket = Trim (Instr (func$, "("))
		
		If bracket
			
			func$ = Left (func$, bracket - 1)
			
			colon = Instr (func$, ":")
			
			If colon
				func$ = Left (func$, colon - 1)
			EndIf
			
		EndIf
		
		WriteLine bmxfile, func$ + " = GetProcAddress (hardwiredlib, " + Chr (34) + fd\dllfunc$ + Chr (34) + ")"
		WriteLine bmxfile, "If Not " + func$ + " Then ListAddLast missing_functions, " + Chr (34) + fd\dllfunc$ + Chr (34)
		WriteLine bmxfile, ""
	Next

	WriteLine bmxfile, "?Debug"
	WriteLine bmxfile, "ListMissingFunctions"
	WriteLine bmxfile, "?"
	WriteLine bmxfile, ""

	CloseFile bmxfile
	
EndIf

Input "BlitzMax conversion done! ENTER to exit..."
End