; Collisions Example 
; ------------------ 

;dxUseDirectX11 True

;dxDebugLoudAndHarsh
dxDebugLoud

dxGraphics3D 640,480,32,2
dxSetBuffer dxBackBuffer() 

; Set collision type values 
type_ground=1 
type_character=2 
type_scenery=3 

camera=dxCreateCamera(0) 
dxRotateEntity camera,45,0,0,True
dxPositionEntity camera,0,15,-10,True

light=dxCreateLight(1,0) 
dxRotateEntity light,45,0,0,True

; Create cube 'ground' 
cube=dxCreateCube(0) 
dxNameEntity cube,"ground"
dxScaleEntity cube,10,10,10,True
dxEntityColor cube,0,127,0 
dxEntityRadius cube,10,10
dxEntityBox cube,-1,-1,-1,2,2,2
dxEntityBox cube,-10,-10,-10,20,20,20
dxEntityType cube,type_ground,False
dxPositionEntity cube,0,-5,0,True

; Create sphere 'character' 
sphere=dxCreateSphere( 32,0 ) 
dxNameEntity sphere,"ball"
dxEntityColor sphere,127,0,0 
dxEntityRadius sphere,1,1
dxEntityType sphere,type_character,False
dxPositionEntity sphere,0,17,0,True

; Enable collisions between type_character and type_ground 
;dxCollisions type_character,type_ground,1,2  ;sphere
dxCollisions type_character,type_ground,2,2  ;polygon
;dxCollisions type_character,type_ground,3,2 ;box

; Create cylinder 'scenery' 
cylinder=dxCreateCylinder( 32,True,0 ) 
dxNameEntity cylinder,"cylinder"
dxScaleEntity cylinder,2,2,2,True
dxEntityColor cylinder,0,0,255 
dxEntityRadius cylinder,2,2
dxEntityBox cylinder,-2,-2,-2,4,4,4 
dxEntityType cylinder,type_scenery,False
dxPositionEntity cylinder,-4,7,-4,True

; Create cone 'scenery' 
cone=dxCreateCone( 32,True,0 ) 
dxNameEntity cone,"cone"
dxScaleEntity cone,2,2,2,True
dxEntityColor cone,0,0,255 
dxEntityRadius cone,2,2 
dxEntityBox cone,-2,-2,-2,4,4,4 
dxEntityType cone,type_scenery,False 
dxPositionEntity cone,4,7,-4,True

; Create prism 'scenery' 
prism=dxCreateCylinder( 3,True,0 ) 
dxNameEntity prism,"prism"
dxScaleEntity prism,2,2,2,True
dxEntityColor prism,0,0,255 
dxEntityRadius prism,2,2
dxEntityBox prism,-2,-2,-2,4,4,4 
dxEntityType prism,type_scenery ,False
dxPositionEntity prism,-4,7,4 ,True
dxRotateEntity prism,0,180,0 ,True

; Create pyramid 'scenery' 
pyramid=dxCreateCone( 4,True,0 ) 
dxNameEntity pyramid,"pyramid"
dxScaleEntity pyramid,2,2,2,True
dxEntityColor pyramid,0,0,255 
dxEntityRadius pyramid,2,2
dxEntityBox pyramid,-2,-2,-2,4,4,4 
dxEntityType pyramid,type_scenery ,False
dxRotateEntity pyramid,0,45,0 ,True
dxPositionEntity pyramid,4,7,4 ,True

; Set collision method and response values 
method=1
response=2 

method_info$="ellipsoid-to-sphere" 
response_info$="slide1" 

While Not dxKeyDown( 1 ) 

x#=0 
y#=0
z#=0 

If dxKeyDown( 203 )=True Then x#=-0.1 
If dxKeyDown( 205 )=True Then x#=0.1 
If dxKeyDown( 208 )=True Then z#=-0.1 
If dxKeyDown( 200 )=True Then z#=0.1 

dxMoveEntity sphere,x#,y#,z# 
dxMoveEntity sphere,0,-0.1,0 ; gravity 

; Change collision method 
If dxKeyHit( 50 )=True 
method=method+1 
If method=4 Then method=1 
If method=1 Then method_info$="ellipsoid-to-sphere" 
If method=2 Then method_info$="ellipsoid-to-polygon" 
If method=3 Then method_info$="ellipsoid-to-box" 
EndIf 

; Change collision response 
If dxKeyHit( 19 )=True 
response=response+1 
If response=4 Then response=1 
If response=1 Then response_info$="stop" 
If response=2 Then response_info$="slide1" 
If response=3 Then response_info$="slide2" 
EndIf 

; Enable collisions between type_character and type_scenery 
dxCollisions type_character,type_scenery,method,response 



; Perform collision checking 
dxUpdateWorld 0



dxRenderWorld 1

;dxMoveEntity camera,0,0,-.004

dxText 0,0,"Use cursor keys to move sphere" ,0,0
dxText 0,20,"Press M to change collision Method (currently: "+method_info$+")" ,0,0
dxText 0,40,"Press R to change collision Response (currently: "+response_info$+")" ,0,0
dxText 0,60,"Collisions type_character,type_scenery,"+method+","+response ,0,0

colls=dxCountCollisions(sphere)
dxText 0,80,"Sphere collided "+colls+" times during last dxupdateworld",0,0
ct=dxEntityCollided(sphere,type_scenery)
For a=1 To colls
dxText 0,80+(a*20),"Collision with "+dxEntityName(dxCollisionEntity(sphere,a))+" !!!",0,0
Next
If ct dxText 0,140,"Collision with "+dxEntityName(ct)+" of entity type "+type_scenery,0,0
If colls
dxText 0,160,"Collision n1 took place at coords "+dxCollisionX(sphere,1)+","+dxCollisionY(sphere,1)+","+dxCollisionZ(sphere,1),0,0
dxText 0,180,"Collision n1 has normal of "+dxCollisionNX(sphere,1)+","+dxCollisionNY(sphere,1)+","+dxCollisionNZ(sphere,1),0,0
dxText 0,200,"Collision n1 has the calc time of "+dxCollisionTime(sphere,1),0,0

EndIf
dxText 0,220,dxGetDebugFloat(),0,0

dxText 0,160,"Collision n1 took place at coords "+dxCollisionX(sphere,1)+","+dxCollisionY(sphere,1)+","+dxCollisionZ(sphere,1),0,0
dxText 0,200,"Collision n1 has the calc time of "+dxCollisionTime(sphere,1),0,0

dxFlip 1


;If dxGetDebugFloat()>0 Stop

Wend 

End  
